unit fonctions_persistent;
// Unité de la Version 2 du projet FormMain
// La version 1 TFormMain n'est pas sa fenêtre parente

// Le module crée des propriété servant à la gestion du fichier INI
// Il gère la déconnexion
// Il gère la gestion des touches majuscules et numlock
// Il gère les forms enfants
// créé par Matthieu Giroux en décembre 2007

{$I ..\DLCompilers.inc}
{$I ..\extends.inc}

interface

{$IFDEF FPC}
{$mode Delphi}
{$ENDIF}

uses
{$IFDEF VERSIONS}
  fonctions_version,
{$ENDIF}
  LazUTF8Classes,
  SysUtils, Variants, Classes;

{$IFDEF VERSIONS}
  const
    gVer_fonctions_persistent : T_Version = (  Component : 'Gestion de Fenêtres' ;
                                       FileUnit : 'fonctions_forms' ;
                                       Owner : 'Matthieu Giroux' ;
                                       Comment : 'Fiche principale deuxième version.' ;
                                       BugsStory : '1.0.1.1 : Simplfying' +#13#10+
                                                   '1.0.1.0 : Resizing from OS' +#13#10+
                                                   '1.0.0.0 : Windows management from FormMainIni.';
                                       UnitType : 3 ;
                                       Major : 1 ; Minor : 0 ; Release : 1 ; Build : 1 );

{$ENDIF}
// Procédure d'enregistrement des forms propres à l'application
procedure p_RegisterClass(AClass: TPersistentClass);
procedure p_RegisterClasses(AClasses: array of TPersistentClass);
procedure p_UnRegisterClass(AClass: TPersistentClass);
procedure p_UnRegisterClasses(AClasses: array of TPersistentClass);
{$IFDEF DELPHI}
procedure p_UnRegisterModuleClasses(Module: HMODULE);
{$ENDIF}
function fper_FindClass(const ClassName: string): TPersistentClass;
function fper_GetClass(const AClassName: string): TPersistentClass;
procedure p_InitRegisterPersistentClasses;
//traductions in FCL
function fs_GetLabelCaption ( const as_Name : String ):WideString;

type
    { TRegGroup }

    TRegGroup = class
    private
      FClassList: TList;
      FAliasList: TStringList;
      FGroupClasses: TList;
      FActive: Boolean;
      function BestClass(AClass: TPersistentClass): TPersistentClass;
    public
      // constrcuteur
      constructor Create(AClass: TPersistentClass);
      // destructeur
      destructor Destroy; override;
      class function BestGroup(Group1, Group2: TRegGroup; AClass: TPersistentClass): TRegGroup;
      // Ajoute un type de classe
      // AClass : type de classe
      procedure AddClass(AClass: TPersistentClass);
      function GetClass(const AClassName: string): TPersistentClass;
  {$IFDEF DELPHI}
      procedure GetClasses(Proc: TGetClass);
      procedure UnregisterModuleClasses(Module: HMODULE);
  {$ENDIF}
      function InGroup(AClass: TPersistentClass): Boolean;
      procedure RegisterClass(AClass: TPersistentClass);
      procedure RegisterClassAlias(AClass: TPersistentClass; const Alias: string);
      function Registered(AClass: TPersistentClass): Boolean;
      procedure UnregisterClass(AClass: TPersistentClass);
      property Active: Boolean read FActive write FActive;
    end;

    TRegGroups = class
    private
      FGroups: TList;
      FLock: TRTLCriticalSection;
      FActiveClass: TPersistentClass;
      function FindGroup(AClass: TPersistentClass): TRegGroup;
    public
      constructor Create;
      destructor Destroy; override;
      procedure Activate(AClass: TPersistentClass);
      procedure AddClass(ID: Integer; AClass: TPersistentClass);
      function GetClass(const AClassName: string): TPersistentClass;
      function GroupedWith(AClass: TPersistentClass): TPersistentClass;
      procedure GroupWith(AClass, AGroupClass: TPersistentClass);
      procedure Lock;
      procedure RegisterClass(AClass: TPersistentClass);
      procedure RegisterClassAlias(AClass: TPersistentClass; const Alias: string);
      function Registered(AClass: TPersistentClass): Boolean;
      procedure StartGroup(AClass: TPersistentClass);
      procedure Unlock;
      procedure UnregisterClass(AClass: TPersistentClass);
  {$IFDEF DELPHI}
      procedure UnregisterModuleClasses(Module: HMODULE);
  {$ENDIF}
      property ActiveClass: TPersistentClass read FActiveClass;
    end;


var
  gb_ModalStarted : Boolean;
//  gb_FreeAllWindowsClosed : Boolean = False ;
  gReg_MainFormIniClassesLocales : TRegGroups = nil ;
  gstl_Labels             : TStringlistUTF8 = nil ;


implementation

uses TypInfo,
     fonctions_string,
     fonctions_system;

{ fonctions }

procedure p_InitRegisterPersistentClasses;
Begin
  gReg_MainFormIniClassesLocales := TRegGroups.Create ;
end;

{ TRegGroup }

// Ajoute un type de classe
// AClass : type de classe
procedure TRegGroup.AddClass(AClass: TPersistentClass);
begin
  FGroupClasses.Add(AClass);
end;

function TRegGroup.BestClass(AClass: TPersistentClass): TPersistentClass;
var
	I: Integer;
	Current: TPersistentClass;
begin
  Result := nil;
  for I := 0 to FGroupClasses.Count - 1 do
  begin
    Current := FGroupClasses[I];
    if AClass.InheritsFrom(Current) then
      if (Result = nil) or Current.InheritsFrom(Result) then
        Result := Current;
  end;
end;

class function TRegGroup.BestGroup(Group1, Group2: TRegGroup;
	AClass: TPersistentClass): TRegGroup;
var
	Group1Class: TPersistentClass;
	Group2Class: TPersistentClass;
begin
  if Group1 <> nil then
    Group1Class := Group1.BestClass(AClass)
   else
    Group1Class := nil;
  if Group2 <> nil then
    Group2Class := Group2.BestClass(AClass)
   else
    Group2Class := nil;
  if Group1Class = nil then
   if Group2Class = nil then
	// AClass is not in either group, no best group
     Result := nil
    else
      // AClass is in Group2 but not Group1, Group2 is best
     Result := Group2
    else
     if Group2Class = nil then
      // AClass is in Group1 but not Group2, Group1 is best
       Result := Group1
      else
       // AClass is in both groups, select the group with the closest ancestor
       if Group1Class.InheritsFrom(Group2Class) then
         Result := Group1
        else
         Result := Group2;
end;

constructor TRegGroup.Create(AClass: TPersistentClass);
begin
  inherited Create;
  FClassList := TList.Create;
  FAliasList := TStringList.Create;
  FGroupClasses := TList.Create;
  FGroupClasses.Add(AClass);
end;

destructor TRegGroup.Destroy;
begin
  inherited Destroy;
  FClassList.Free;
  FAliasList.Free;
  FGroupClasses.Free;
end;

function TRegGroup.GetClass(const AClassName: string): TPersistentClass;
var
	I: Integer;
begin
  if FClassList <> nil then
    for I := 0 to FClassList.Count - 1 do
    begin
      Result := FClassList[I];
      if Result.ClassNameIs(AClassName) then Exit;
    end;
  if FAliasList <> nil then
  begin
    I := FAliasList.IndexOf(AClassName);
    if I >= 0 then
    begin
      Result := TPersistentClass(FAliasList.Objects[I]);
      Exit;
    end;
  end;
  Result := nil;

end;

{$IFDEF DELPHI}
procedure TRegGroup.GetClasses(Proc: TGetClass);
var
  I: Integer;
begin
  for I := 0 to FClassList.Count - 1 do
    Proc(TPersistentClass(FClassList[I]));
end;
{$ENDIF}

function TRegGroup.InGroup(AClass: TPersistentClass): Boolean;
var
  I: Integer;
begin
  Result := True;
  for I := 0 to FGroupClasses.Count - 1 do
    if AClass.InheritsFrom(TPersistentClass(FGroupClasses[I])) then Exit;
  Result := False;
end;

procedure TRegGroup.RegisterClass(AClass: TPersistentClass);
var
  LClassName: string;
begin
  LClassName := AClass.ClassName;
  if GetClass(LClassName) = nil then
    FClassList.Add(AClass);
end;

procedure TRegGroup.RegisterClassAlias(AClass: TPersistentClass;
  const Alias: string);
begin
  RegisterClass(AClass);
  FAliasList.AddObject(Alias, TObject(AClass));
end;

function TRegGroup.Registered(AClass: TPersistentClass): Boolean;
begin
  Result := FClassList.IndexOf(AClass) >= 0;
end;

procedure TRegGroup.UnregisterClass(AClass: TPersistentClass);
var
  I: Integer;
begin
  FClassList.Remove(AClass);
  for I := FAliasList.Count - 1 downto 0 do
    if FAliasList.Objects[I] = TObject(AClass) then
      FAliasList.Delete(I);
end;

{$IFDEF DELPHI}


function PointerInModule(Ptr: Pointer; Module: HMODULE): Boolean;
begin
  Result := (Module = 0) or (FindHInstance(Ptr) = Module);
end;

procedure TRegGroup.UnregisterModuleClasses(Module: HMODULE);
var
  I: Integer;
begin
  // Even though the group criterion changes we do not need to recalculate the
  // groups because the groups are based on ancestry. If an ancestor of a class
  // is removed because its module was unloaded we can safely assume that all
  // descendents have also been unloaded or are being unloaded as well. This
  // means that any classes left in the registry are not descendents of the
  // classes being removed and, therefore, will not be affected by the change
  // to the FGroupClasses list.
  for I := FGroupClasses.Count - 1 downto 0 do
    if PointerInModule(FGroupClasses[I], Module) then
      FGroupClasses.Delete(I);
	for I := FClassList.Count - 1 downto 0 do
    if PointerInModule(FClassList[I], Module) then
      FClassList.Delete(I);
  for I := FAliasList.Count - 1 downto 0 do
    if PointerInModule(FAliasList.Objects[I], Module) then
      FAliasList.Delete(I);
end;

{$ENDIF}

{ TRegGroups }

procedure TRegGroups.Activate(AClass: TPersistentClass);
var
  I: Integer;
begin
  if FActiveClass <> AClass then
  begin
    FActiveClass := AClass;
    for I := 0 to FGroups.Count - 1 do
      with TRegGroup(FGroups[I]) do
        Active := InGroup(AClass);
  end;
end;

procedure TRegGroups.AddClass(ID: Integer; AClass: TPersistentClass);
begin
  TRegGroup(FGroups[ID]).AddClass(AClass);
end;

constructor TRegGroups.Create;
var
  Group: TRegGroup;
begin
  inherited Create;
  FGroups := TList.Create;
{$IFDEF FPC}InitCriticalSection{$ELSE}InitializeCriticalSection{$ENDIF}(FLock);
	// Initialize default group
  Group := TRegGroup.Create(TPersistent);
  FGroups.Add(Group);
  Group.Active := True;
end;

destructor TRegGroups.Destroy;
var
  I: Integer;
begin
  {$IFDEF FPC}System.LeaveCriticalSection{$ELSE}DeleteCriticalSection{$ENDIF}(FLock);
  for I := 0 to FGroups.Count - 1 do
    TRegGroup(FGroups[I]).Free;
  FGroups.Free;
  inherited;
end;

function TRegGroups.FindGroup(AClass: TPersistentClass): TRegGroup;
var
  I: Integer;
  Current: TRegGroup;
begin
  Result := nil;
  for I := 0 to FGroups.Count - 1 do
  begin
    Current := TRegGroup ( FGroups[I] );
    Result := TRegGroup.BestGroup(Current, Result, AClass);
  end;
end;

function TRegGroups.GetClass(const AClassName: string): TPersistentClass;
var
  I: Integer;
begin
  Result := nil;
	for I := 0 to FGroups.Count - 1 do
    with TRegGroup(FGroups[I]) do
    begin
      if Active then Result := GetClass(AClassName);
      if Result <> nil then Exit;
    end;
end;

function TRegGroups.GroupedWith(AClass: TPersistentClass): TPersistentClass;
var
  Group: TRegGroup;
begin
  Result := nil;
  Group := FindGroup(AClass);
	if Group <> nil then
    Result := TPersistentClass(Group.FGroupClasses[0]);
end;

procedure TRegGroups.GroupWith(AClass, AGroupClass: TPersistentClass);

  procedure Error;
  begin
//    raise EFilerError.CreateFmt(SUnknownGroup, [AGroupClass.ClassName]);
  end;

var
  Group: TRegGroup;
  CurrentGroup: TRegGroup;
  CurrentClass: TPersistentClass;
  I, J: Integer;
begin
  Group := FindGroup(AGroupClass);
  if Group = nil then Error;
  Group.AddClass(AClass);

	// The group criterion has changed. We need to recalculate which groups the
  // classes that have already been registered belong to. We can skip
  // Group since we would just be moving a class to a group it already belongs
  // to. We also only need to find the new group of classes that descend from
  // AClass since that is the only criterion being changed. In other words,
  // we only need to move classes that descend from AClass to Group if they
  // are in another group.
  for I := 0 to FGroups.Count - 1 do
  begin
    CurrentGroup := TRegGroup ( FGroups[I]^ );
    if CurrentGroup <> Group then
      for J := CurrentGroup.FClassList.Count - 1 downto 0 do
      begin
	CurrentClass := TPersistentClass ( CurrentGroup.FClassList[J]^ );
        if CurrentClass.InheritsFrom(AClass) then
					// Check CurrentClass should be put into Group based on the new
          // criterion. Their might be a descendent of AClass registered that
          // overrides Group's criterion.
          if FindGroup(CurrentClass) = Group then
          begin
            CurrentGroup.FClassList.Delete(J);
            Group.FClassList.Add(CurrentClass);
          end;
      end;
  end;
end;

procedure TRegGroups.Lock;
begin
{$IFDEF FPC}
  System.
{$ENDIF}
  EnterCriticalSection(FLock);
end;

procedure TRegGroups.RegisterClass(AClass: TPersistentClass);
var
  Group: TRegGroup;
begin
  Group := FindGroup(AClass);
  if Group <> nil then Group.RegisterClass(AClass);
end;

procedure TRegGroups.RegisterClassAlias(AClass: TPersistentClass;
  const Alias: string);
var
  Group: TRegGroup;
begin
  Group := FindGroup(AClass);
  if Group <> nil then Group.RegisterClassAlias(AClass, Alias);
end;

function TRegGroups.Registered(AClass: TPersistentClass): Boolean;
var
  I: Integer;
begin
  Result := True;
  for I := 0 to FGroups.Count - 1 do
    if TRegGroup(FGroups[I]).Registered(AClass) then Exit;
  Result := False;
end;

procedure TRegGroups.StartGroup(AClass: TPersistentClass);
var
  I: Integer;
begin
  // Do not start a group that already exists
  for I := 0 to FGroups.Count - 1 do
    if TRegGroup(FGroups[I]).FGroupClasses.IndexOf(AClass) >= 0 then
      Exit;
	// Create the group
  FGroups.Add(TRegGroup.Create(AClass));
end;

procedure TRegGroups.Unlock;
begin
{$IFDEF FPC}
  System.
{$ENDIF}
  EnterCriticalSection(FLock);
end;

procedure TRegGroups.UnregisterClass(AClass: TPersistentClass);
var
  I: Integer;
begin
  for I := 0 to FGroups.Count - 1 do
    TRegGroup(FGroups[I]).UnregisterClass(AClass);
end;

{$IFDEF DELPHI}

procedure TRegGroups.UnregisterModuleClasses(Module: HMODULE);
var
  I: Integer;
  Group: TRegGroup;
begin
  for I := FGroups.Count - 1 downto 0 do
  begin
    Group := TRegGroup(FGroups[I]);
    Group.UnregisterModuleClasses(Module);
    if Group.FGroupClasses.Count = 0 then
    begin
      Group.Free;
      FGroups.Delete(I);
    end;
  end;
end;

procedure p_UnRegisterModuleClasses(Module: HMODULE);
begin
  gReg_MainFormIniClassesLocales.Lock;
  try
    gReg_MainFormIniClassesLocales.UnregisterModuleClasses(Module);
  finally
    gReg_MainFormIniClassesLocales.Unlock;
	end;
end;
{$ENDIF}

  // Procédure d'enregistrement d'une forms propre à l'application
  procedure p_RegisterClass(AClass: TPersistentClass);
begin
  gReg_MainFormIniClassesLocales.Lock;
  try
    while not gReg_MainFormIniClassesLocales.Registered(AClass) do
     begin
      gReg_MainFormIniClassesLocales.RegisterClass(AClass);
      if AClass = TPersistent then Break;
      AClass := TPersistentClass(AClass.ClassParent);
     end;
  finally
    gReg_MainFormIniClassesLocales.Unlock;
  end;
end;

	// Procédure d'enregistrement des forms propres à l'application
procedure p_RegisterClasses(AClasses: array of TPersistentClass);
var
  I: Integer;
begin
  for I := Low(AClasses) to High(AClasses) do p_RegisterClass(AClasses[I]);
end;

procedure p_RegisterClassAlias(AClass: TPersistentClass; const Alias: string);
begin
  gReg_MainFormIniClassesLocales.Lock;
  try
    gReg_MainFormIniClassesLocales.RegisterClassAlias(AClass, Alias);
  finally
    gReg_MainFormIniClassesLocales.Unlock;
  end;
end;

procedure p_UnRegisterClass(AClass: TPersistentClass);
begin
  gReg_MainFormIniClassesLocales.Lock;
  try
    gReg_MainFormIniClassesLocales.UnregisterClass(AClass);
  finally
    gReg_MainFormIniClassesLocales.Unlock;
  end;
end;

procedure p_UnRegisterClasses(AClasses: array of TPersistentClass);
var
  I: Integer;
begin
  for I := Low(AClasses) to High(AClasses) do Classes.UnRegisterClass(AClasses[I]);
end;

function fper_GetClass(const AClassName: string): TPersistentClass;
begin
  gReg_MainFormIniClassesLocales.Lock;
  try
    Result := gReg_MainFormIniClassesLocales.GetClass(AClassName);
  finally
    gReg_MainFormIniClassesLocales.Unlock;
  end;
end;

function fper_FindClass(const ClassName: string): TPersistentClass;
begin
  Result := fper_GetClass(ClassName);
end;
// function fs_GetLabelCaption
// Getting label caption from name
// Name of caption
function fs_GetLabelCaption ( const as_Name : String ):WideString;
Begin
   Result := fs_GetStringValue ( gstl_Labels, as_Name );
end;

initialization
{$IFDEF VERSIONS}
  p_ConcatVersion ( gVer_fonctions_persistent );
{$ENDIF}
finalization
  FreeAndNil(gstl_Labels);
  FreeAndNil(gReg_MainFormIniClassesLocales);
end.
