﻿unit u_exttablesgrid;

{*********************************************************************}
{                                                                     }
{             TExtDBGrid :                                            }
{             Grille avec couleurs de focus, d'édition,               }
{             permettant d'être éditée avec des composants de données }
{             Créateur : Matthieu Giroux                              }
{             13 Juin 2011                                            }
{                                                                     }
{*********************************************************************}

{$I ..\DLCompilers.inc}
{$I ..\extends.inc}
{$IFDEF FPC}
{$mode Delphi}
{$ENDIF}

interface

uses
{$IFDEF VERSIONS}
  fonctions_version,
{$ENDIF}
  Classes, SysUtils, Grids,
  u_extcomponent, Graphics,
  DB, u_extdbgrid,
  fonctions_db,
  fonctions_manbase,
  fonctions_createsql,
  DBGrids,
  Controls;

{$IFDEF VERSIONS}
const
    gVer_ExtTablesGrid : T_Version = ( Component : 'Grid with Crossed Request' ;
                                               FileUnit : 'U_ExtTablesGrid' ;
                                               Owner : 'Matthieu Giroux' ;
                                               Comment : 'Grid with Crossed Request' ;
                                               BugsStory : '0.9.0.0 : Creating from ExtDbGrid.' ;
                                               UnitType : 3 ;
                                               Major : 0 ; Minor : 9 ; Release : 0 ; Build : 0 );
{$ENDIF}


   { TExtGridColumn }
type

   { TExtTablesGrid }

   TExtTablesGrid = class ( TExtDBGrid )
     private
       FTables : TFWTables;
       function  GetTables : TFWTables;
       procedure SetTables ( const ATables : TFWTables );
       function  GetDataset : TDataset;
      protected
        function CreateTables:TFWTables;
      protected
        procedure SetSQLQuery; virtual;
        procedure SetDatasource; virtual;
      public
       property Datasource;
       property Dataset : TDataset read GetDataset;
       constructor Create ( AComponent : TComponent ); override;
       procedure Loaded; override;
      published
       property Tables: TFWTables read GetTables write SetTables;
     End;

implementation


uses
{$IFNDEF FPC}
   Forms,
{$ENDIF}
   fonctions_proprietes,
   Menus,
   u_multidonnees,
   fonctions_dbobjects,
   fonctions_images;

{ TExtTablesGrid }

function TExtTablesGrid.GetTables: TFWTables;
begin
  Result:=FTables;
end;

procedure TExtTablesGrid.SetTables(const ATables: TFWTables);
begin
  FTables.Assign(ATables);
end;

function TExtTablesGrid.GetDataset: TDataset;
begin
  if Assigned(DataSource)
   Then Result:=DataSource.DataSet
   Else Result:=nil;
end;

function TExtTablesGrid.CreateTables: TFWTables;
begin
  Result:=TFWTables.Create(Self,TFWTable);
end;

constructor TExtTablesGrid.Create(AComponent: TComponent);
begin
  inherited Create(AComponent);
  FTables:=CreateTables;
end;

procedure TExtTablesGrid.SetDatasource;
var lds_Datasource : TDatasource;
    ldat_Dataset : TDataSet;
Begin
  if not ( csDesigning in ComponentState )
  and ( DMModuleSources.Sources.Count > 0 ) Then
   Begin
     if not Assigned(DataSource) Then
      Begin
        lds_Datasource := nil;
        ldat_Dataset := fdat_CloneDatasetWithoutSQLWithDataSource(DMModuleSources.Sources [ 0 ].QueryCopy,Owner,lds_Datasource);
        DataSource:=lds_Datasource;
      End;
   end;
  SetSQLQuery;
  DataSource.DataSet.Open;
end;

procedure TExtTablesGrid.Loaded;
begin
  inherited Loaded;
  SetDatasource;
end;


procedure TExtTablesGrid.SetSQLQuery;
begin
  p_SetSQLQuery(Dataset,FTables.fs_GetEveryLinkedTables);
end;

{$IFDEF VERSIONS}
initialization
  // Gestion de version
  p_ConcatVersion(gVer_ExtTablesGrid);
{$ENDIF}
end.

